
(function (root, factory) {
    if (typeof exports === 'object') {
        module.exports = factory(require('react'));
    } else if (typeof define === 'function' && define.amd) {
        define(['react'], factory);
    } else {
        // This part I think it only makes sense while using <script> to include the libs
        root.unexpectedReact = factory(root.react);
    }
}(this, function (React) {
    return {
        name: 'unexpected-react',
        installInto: function (expect) {

            expect.addType({
                name: 'ReactElement',
                identify: function (obj) {
                  return !!(React.addons.TestUtils.isElement(obj) || (obj && obj.props && obj.state && obj.refs))
                },
                // The inspect still needs to be defined correctly
                inspect: function (value, depth, output, inspect) { output.text('<React.js component...>'); }
            });

            // Checks if the element has certain props (not the value)
            // expect(component, 'to have props', ['values', 'label', 'showLabel', 'size']);
            // Can either match against an array or an object
            expect.addAssertion('ReactElement', '[not] to have props', function (expect, subject, properties) {
                expect(subject.props, '[not] to have properties', properties);
            });

            // Checks if the element has certain props (not the value)
            // expect(component, 'to have props', ['values', 'label', 'showLabel', 'size']);
            // Can either match against an array or an object
            expect.addAssertion('ReactElement', '[not] to have state', function (expect, subject, properties) {
                expect(subject.state, '[not] to have properties', properties);
            });

            // Checks if the element has a certain amount of children
            expect.addAssertion('ReactElement', 'to have children', function (expect, subject, childrenCount) {

              var subjectChildren;

              if (subject.props.children) {
                if (subject.props.children.length) {
                  subjectChildren = subject.props.children.length;
                }
                else {
                  subjectChildren = 1;
                }
              }
              else {
                subjectChildren = undefined;
              }

              expect(subjectChildren, 'to be', childrenCount);
            });

            // Checks if the element has no children
            expect.addAssertion('ReactElement', 'not to have children', function (expect, subject) {
                var children = subject.props.children;
                expect(children, 'to be undefined');
            });

        }
    };
}));
